//
//  PenColorView.m
//  color
//
//  Created by kunii on 10/01/14.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "PenColorView.h"


@implementation PenColorView
@synthesize delegate;

static void drawRoundRect(
	CGContextRef		inContext,
	int					inWidth,
	int					inHeight,
	CGPathDrawingMode	inMode)
{
	CGFloat radius = 10.0;
	CGFloat minx = 0, midx = inWidth / 2, maxx = inWidth;
	CGFloat miny = 0, midy = inHeight / 2, maxy = inHeight;
	CGContextMoveToPoint(inContext, minx, midy);
	CGContextAddArcToPoint(inContext, minx, miny, midx, miny, radius);
	CGContextAddArcToPoint(inContext, maxx, miny, maxx, midy, radius);
	CGContextAddArcToPoint(inContext, maxx, maxy, midx, maxy, radius);
	CGContextAddArcToPoint(inContext, minx, maxy, minx, midy, radius);
	CGContextClosePath(inContext);
	CGContextDrawPath(inContext, inMode);
}

/*
	グラデーション画像を作成し返す。
*/
static CGImageRef createMaskImage(
	int inWidth,
	int inHeight)
{
	CGImageRef theCGImage = NULL;		//	作成されたグラデーション画像。
	//	グラデーションは白黒の諧調なので色空間はグレーとする。
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceGray();
	//	画像バッファ作成。
	CGContextRef maskBitmapContext = CGBitmapContextCreate(
		NULL, inWidth, inHeight,
		8, 0, colorSpace, kCGImageAlphaNone);
	if (maskBitmapContext != NULL) {
		//	描画。
		CGContextSetGrayFillColor(maskBitmapContext, 0.0, 1.0);
		CGContextFillRect(maskBitmapContext, CGRectMake(0, 0, inWidth, inHeight));
		CGContextSetGrayStrokeColor(maskBitmapContext, 1.0, 1.0);
		CGContextSetGrayFillColor(maskBitmapContext, 1.0, 1.0);
		CGContextSetLineWidth(maskBitmapContext, 2.0);
		drawRoundRect(maskBitmapContext, inWidth, inHeight, kCGPathFillStroke);
		//	画像バッファをCGImageRefに変換。
		theCGImage = CGBitmapContextCreateImage(maskBitmapContext);
		//	画像バッファに描画するためのコンテキストを解放。
		CGContextRelease(maskBitmapContext);
		
	}
	//	色空間情報を解放。
	CGColorSpaceRelease(colorSpace);
	//	グラデーション画像を返す。
    return theCGImage;
}

/*
	グラデーション画像を作成し返す。
*/
static CGImageRef createGradientImage(
	int			inWidth,
	int			inHeight,
	UIColor*	inStartColor,
	UIColor*	inEndColor)
{
	CGImageRef theCGImage = NULL;		//	作成されたグラデーション画像。
	//	グラデーションは白黒の諧調なので色空間はグレーとする。
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
	//	画像バッファ作成。
	CGContextRef gradientBitmapContext = CGBitmapContextCreate(
		NULL, inWidth, inHeight,
		8, 0, colorSpace, kCGImageAlphaPremultipliedLast);
	if (gradientBitmapContext != NULL) {
		//	バッファが作成できたのでグラデーションをほどこす。
		const CGFloat locations[] = {
			0.0,
			1.0
		};
		CGColorRef colors[] = {
			inStartColor.CGColor,
			inEndColor.CGColor
		};
		CFArrayRef array = CFArrayCreate(kCFAllocatorDefault, 
			(const void **)colors, 2, NULL);
		//	グラデーション情報の作成。
		CGGradientRef colorGradient
			= CGGradientCreateWithColors(colorSpace, array, locations);
		CFRelease(array);
		//	このグラデーションを、画像バッファのどの範囲に割り付けるかを設定。
		CGPoint gradientStartPoint = CGPointMake(0,inHeight);
		CGPoint gradientEndPoint = CGPointZero;
		//	描画。
		CGContextDrawLinearGradient(gradientBitmapContext, colorGradient, 
			gradientStartPoint, gradientEndPoint, kCGGradientDrawsAfterEndLocation);
		//	グラデーション情報は用が無くなったので解放。
		CGGradientRelease(colorGradient);

		CGContextSetRGBStrokeColor(gradientBitmapContext, 1.0, 1.0, 1.0, 1.0);
		CGContextSetLineWidth(gradientBitmapContext, 2.0);
		drawRoundRect(gradientBitmapContext, inWidth, inHeight, kCGPathStroke);

		//	画像バッファをCGImageRefに変換。
		theCGImage = CGBitmapContextCreateImage(gradientBitmapContext);
		//	画像バッファに描画するためのコンテキストを解放。
		CGContextRelease(gradientBitmapContext);
		
	}
	//	色空間情報を解放。
	CGColorSpaceRelease(colorSpace);
	//	グラデーション画像を返す。
    return theCGImage;
}


- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
		self.backgroundColor = [UIColor clearColor];
		startColor = [UIColor colorWithHue:0.0 saturation:1.0 brightness:1.0 alpha:1.0];
		[startColor retain];
		endColor = [UIColor colorWithHue:0.3 saturation:1.0 brightness:1.0 alpha:1.0];
		[endColor retain];
    }
    return self;
}


- (void)drawRect:(CGRect)rect {
	if (image == nil) {
		CGRect r = self.bounds;
		CGImageRef gradientCGImageRef = createGradientImage(r.size.width, r.size.height, startColor, endColor);
		CGImageRef maskCGImageRef = createMaskImage(r.size.width, r.size.height);
		CGImageRef cgimageRef = CGImageCreateWithMask(gradientCGImageRef, maskCGImageRef);
		image = [UIImage imageWithCGImage:cgimageRef];
		CGImageRelease(gradientCGImageRef);
		CGImageRelease(maskCGImageRef);
		CGImageRelease(cgimageRef);
	}
	[image drawAtPoint:CGPointMake(0, 0)];
}


- (void)dealloc {
	[image release];
	[startColor release];
	[endColor release];
    [super dealloc];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	//	マルチタッチを許していないので、必ずワンフィンガー
    if ([delegate respondsToSelector:@selector(penColorView:color:)]) {
		UITouch *touch = [touches anyObject];	//	なので、一つ取り出せばよい。
		CGPoint tapLocation = [touch locationInView:self];
		CGRect r = self.bounds;

		double value = tapLocation.y / r.size.height;		//	0.0〜1.0の範囲に正規化
		//	startColor、endColorの色要素取り出し。
		const CGFloat* startComponents = CGColorGetComponents(startColor.CGColor);
		const CGFloat* endComponents = CGColorGetComponents(endColor.CGColor);
		//	グラデーションはリニアに指定しているのでこの計算でだいたい一致する。
		UIColor* color = [UIColor colorWithRed:(endComponents[0] - startComponents[0]) * value + startComponents[0]
			green:(endComponents[1] - startComponents[1]) * value + startComponents[1]
			blue:(endComponents[2] - startComponents[2]) * value + startComponents[2]
			alpha:1.0];
        [delegate penColorView:self color:color];
	}
}

//	両端の色を指定する。このタイミングでいったんimageをnilに戻す。
-(void)setColorStart:(UIColor*)inStartColor end:(UIColor*)inEndColor {
	[image release];
	image = nil;
	[startColor release];
	startColor = [inStartColor retain];
	[endColor release];
	endColor = [inEndColor retain];
}
@end
