//
//  PenSizeView.m
//  color
//
//  Created by kunii on 10/01/14.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "PenSizeView.h"


@implementation PenSizeView
@synthesize delegate;

- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
		self.backgroundColor = [UIColor clearColor];
		color = [UIColor colorWithHue:0.0 saturation:1.0 brightness:1.0 alpha:1.0];
		[color retain];
    }
    return self;
}

/*
	inImageのマスク情報だけ残してRGB画像だけ変更したUIImageを返す。
*/
static UIImage* drawWithoutAlpha(UIImage* inImage, UIColor* color)
{
	CGImageRef cgImage = inImage.CGImage;
    //	画像情報を得る。
	int width = CGImageGetWidth(cgImage);
	int height = CGImageGetHeight(cgImage);
	int bitsPerComponent = CGImageGetBitsPerComponent(cgImage);
	int bitsPerPixel = CGImageGetBitsPerPixel(cgImage);
	int bytesPerRow = CGImageGetBytesPerRow(cgImage);
	CGColorSpaceRef colorSpace = CGImageGetColorSpace(cgImage);
	CGBitmapInfo bitmapInfo = CGImageGetBitmapInfo(cgImage);
	bool shouldInterpolate = CGImageGetShouldInterpolate(cgImage);
    CGColorRenderingIntent intent= CGImageGetRenderingIntent(cgImage);
	//	処理としては必要ないけど、各情報をコンソールに表示しときます。
	printf("width=%d height=%d, bitsPerComponent=%d, bitsPerPixel=%d bytesPerRow=%d\n", 
		width, height, bitsPerComponent, bitsPerPixel, bytesPerRow);
	printf("NumberOfComponents=%d\nModel=%d\n", 
		(int)CGColorSpaceGetNumberOfComponents(colorSpace), 
		CGColorSpaceGetModel(colorSpace));
	printf("bitmapInfo=%d shouldInterpolate=%s intent=%d\n", 
		bitmapInfo, shouldInterpolate?"YES":"NO", intent);
	//	データプロバイダを取り出す。
    CGDataProviderRef dataProvider = CGImageGetDataProvider(cgImage);
    //	画像データを複製する。
	CFDataRef data = CGDataProviderCopyData(dataProvider);

#if 0	//	こっちは直接描画版
	CGContextRef duplicatedContext = CGBitmapContextCreate(
		(void*)CFDataGetBytePtr(data), width, height,
		bitsPerComponent, bytesPerRow, colorSpace, 
		kCGImageAlphaPremultipliedLast);
	CGContextSetLineWidth(duplicatedContext, 5.0);
	CGContextSetRGBStrokeColor(duplicatedContext, 1.0, 0.0, 0.0, 1.0);
	CGContextStrokeEllipseInRect(duplicatedContext, CGRectMake(10, 10, 40, 40));

    //	複製画像データのプロバイダを作成する
    CGDataProviderRef duplicatedDataProvider = CGDataProviderCreateWithCFData(data);
	CFRelease(data);
    CGImageRef duplicatedCgImage = CGImageCreate(
            width, height, 
            bitsPerComponent, bitsPerPixel, bytesPerRow, 
            colorSpace, bitmapInfo, duplicatedDataProvider, 
            NULL, shouldInterpolate, intent);
	CGContextRelease(duplicatedContext);
#else
	//	作業用RGBコンテキストを用意。
	UInt8* workData = (UInt8*)malloc(bytesPerRow * height);
	CGContextRef workContext = CGBitmapContextCreate(
		workData, width, height,
		bitsPerComponent, bytesPerRow, colorSpace, 
		kCGImageAlphaPremultipliedLast);
	//	指定色で塗りつぶし。
	const CGFloat* startComponents = CGColorGetComponents(color.CGColor);
	CGContextSetRGBFillColor(workContext, startComponents[0], startComponents[1], startComponents[2], 1.0);
	CGContextFillRect(workContext, CGRectMake(0,0,width,height));
    //	作業用RGBコンテキストのデータを、複製画像データにアルファチャネルをはぶいてコピー
	UInt8* dupData = (UInt8*)CFDataGetBytePtr(data);
	for (int v = 0; v < height; v++) {
		UInt8* dst = dupData + v * bytesPerRow;
		UInt8* src = workData + v * bytesPerRow;
		for (int h = 0; h < width; h++) {
            *dst++ = *src++;	//	赤コピー
            *dst++ = *src++;	//	緑コピー
            *dst++ = *src++;	//	青コピー
			dst++;	//	αはスキップ
			src++;	//	αはスキップ
        }
    }
	CGContextRelease(workContext);
	free(workData);
    // 複製画像データのデータプロバイダを作成する
    CGDataProviderRef duplicatedDataProvider = CGDataProviderCreateWithCFData(data);
	CFRelease(data);
    CGImageRef duplicatedCgImage = CGImageCreate(
            width, height, 
            bitsPerComponent, bitsPerPixel, bytesPerRow, 
            colorSpace, bitmapInfo, duplicatedDataProvider, 
            NULL, shouldInterpolate, intent);
	CGDataProviderRelease(duplicatedDataProvider);
#endif
	UIImage* duplicatedImage = [[UIImage alloc] initWithCGImage:duplicatedCgImage];
	CGImageRelease(duplicatedCgImage);
	return duplicatedImage;
}

- (void)drawRect:(CGRect)rect {
	if (image == nil) {
		image = drawWithoutAlpha([UIImage imageNamed:@"pen.png"], color);
	}
	[image drawInRect:self.bounds];
}


- (void)dealloc {
	[image release];
	[color release];
    [super dealloc];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	//	マルチタッチを許していないので、必ずワンフィンガー
    if ([delegate respondsToSelector:@selector(penColorView:color:)]) {
		UITouch *touch = [touches anyObject];	//	なので、一つ取り出せばよい。
		CGPoint tapLocation = [touch locationInView:self];
		CGRect r = self.bounds;

		double value = tapLocation.y / r.size.height;		//	0.0〜1.0の範囲に正規化
        [delegate penSizeView:self size:value];
	}
}

//	両端の色を指定する。このタイミングでいったんimageをnilに戻す。
-(void)setPenColor:(UIColor*)inColor {
	[image release];
	image = nil;
	[color release];
	color = [inColor retain];
	[self setNeedsDisplay];
}
@end
